#Region ;**** Directives created by AutoIt3Wrapper_GUI ****
#AutoIt3Wrapper_Change2CUI=y
#AutoIt3Wrapper_Run_After="%out%"
#EndRegion ;**** Directives created by AutoIt3Wrapper_GUI ****
;~ #AutoIt3Wrapper_UseX64=y
#include "Registry.au3"

Global Const $ERROR_SUCCESS = 0

main ( )

;=============================================================================
;-- RecurseEnumKeys - Enumerates recursively the subkeys of key.
;--            hKey - Key whose subkeys are to be enumerated recursively.
func RecurseEnumKeys ($hKey)

	Local $dwIndex = 0
	Local $sKeyName, $hSubKey

	Local $fTime = DllStructCreate ($FILETIME) ;      // last write time
	Local $lpfTime = DllStructGetPtr ($fTime)  ;      // last write time

   while (RegEnumKeyEx ($hKey, $dwIndex, $sKeyName, $NULL, $lpfTime) == $ERROR_SUCCESS)

       printf ("> %s\n", $sKeyName)

       $dwIndex += 1

       if (RegOpenKeyEx ($hKey, $sKeyName, $KEY_READ, $hSubKey) == $ERROR_SUCCESS) then

          RecurseEnumKeys ($hSubKey)

          RegCloseKey ($hSubKey)

       endif
   wend

endfunc

func RecurseEnumValue ($hKey)

	Local $dwIndex = 0
	Local $sKeyName, $hSubKey

	Local $fTime = DllStructCreate ($FILETIME) ;      // last write time
	Local $lpfTime = DllStructGetPtr ($fTime)  ;      // last write time

   while (RegEnumKeyEx ($hKey, $dwIndex, $sKeyName, $NULL, $lpfTime) == $ERROR_SUCCESS)

       printf ("> %s\n", $sKeyName)

       $dwIndex += 1

       if (RegOpenKeyEx ($hKey, $sKeyName, $KEY_READ, $hSubKey) == $ERROR_SUCCESS) then

          RecurseEnumKeys ($hSubKey)

          RegCloseKey ($hSubKey)

       endif
   wend

endfunc

;=============================================================================
;-- QueryKey - Enumerates the subkeys of key and its associated values.
;--     hKey - Key whose subkeys and values are to be enumerated.
func QueryKey ($hKey)

	local $sKey;                      // buffer for subkey name
	local $cbName;                    // size of name string
	local $sClass;                    // buffer for class name
	local $cSubKeys;                  // number of subkeys
	local $cbMaxSubKey;               // longest subkey size
	local $cchMaxClass;               // longest class string
	local $cValues;              // number of values for key
	local $cchMaxValue;          // longest value name
	local $cbMaxValueData;       // longest value data
	local $cbSecurityDescriptor; // size of security descriptor
	local $ftLastWriteTime = DllStructCreate ($FILETIME) ;      // last write time
	local $lpftLastWriteTime = DllStructGetPtr ($ftLastWriteTime) ;      // last write time

	local $i, $retCode

	local $sValue

    ;-- Get the class name and the value count.
    $retCode = RegQueryInfoKey ( _
			   $hKey, _ ;                   // key handle
			   $sClass, _ ;                 // buffer for class name
			   $cSubKeys, _ ;               // number of subkeys
			   $cbMaxSubKey, _ ;            // longest subkey size
			   $cchMaxClass, _ ;            // longest class string
			   $cValues, _ ;                // number of values for this key
			   $cchMaxValue, _ ;            // longest value name
			   $cbMaxValueData, _ ;         // longest value data
			   $cbSecurityDescriptor, _ ;   // security descriptor
			   $lpftLastWriteTime _;         // last write time
			   )
    ;-- Enumerate the subkeys, until RegEnumKeyEx fails.

    if ($cSubKeys) then

        printf ( "+>Number of subkeys: %d\n", $cSubKeys)

        for $i = 0 to $cSubKeys - 1

            $retCode = RegEnumKeyEx ($hKey, $i, _
                     $sKey, _
                     $NULL, _
                     $lpftLastWriteTime)
            if ($retCode == $ERROR_SUCCESS) then _
                printf (">%4d) %s\n", $i+1, $sKey)
        next
	else
		printf ( "!>Number of subkeys: %d\n", $cValues)
    endif

    ;-- Enumerate the key values.

    if ($cValues) then

        printf ( "+>Number of values: %d\n", $cValues)

        for $i = 0 to $cValues - 1

            $retCode = RegEnumValue ($hKey, $i, _
                $sValue, _
                $NULL, _
                $NULL, _
                $NULL)

            if ($retCode == $ERROR_SUCCESS ) then _
                printf (">%4d) %s\n", $i+1, $sValue)
		next
	else
		printf ( "!>Number of values: %d\n", $cValues)
    endif

endfunc

func main ( )

	local $hTestKey

   ;-- einfach
   if ( RegOpenKeyEx ($HKEY_CURRENT_USER, _
        "Software\Classes", _
        $KEY_ALL_ACCESS, _
        $hTestKey) == $ERROR_SUCCESS _
      ) then

      printf ( "-> ### QueryKey ###\n")

      QueryKey ($hTestKey)
	  RegCloseKey ($hTestKey)

      printf ( "--------------------------\n\n")

   endif

   ;-- rekursiv
   if ( RegOpenKeyEx ($HKEY_LOCAL_MACHINE, _
		"SOFTWARE\Classes\AutoIt3Script", _
		$KEY_ALL_ACCESS, _
		$hTestKey) == $ERROR_SUCCESS _
	 ) then

      printf ( "-> ### RecurseEnumKeys ###\n")

      RecurseEnumKeys ($hTestKey)
      RegCloseKey ($hTestKey)

      printf ( "--------------------------\n\n")

   endif

endfunc

func printf ($format, $var1 = '', $var2 = '')
	return ConsoleWrite (StringFormat($format, $var1, $var2))
endfunc

